/* KLook
 * Copyright (c) 2011-2012 ROSA  <support@rosalab.ru>
 * Authors: Julia Mineeva, Evgeniy Auzhin, Sergey Borovkov.
 * License: GPLv3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "klookapp.h"

#include <QDebug>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QGraphicsObject>

#include <KCmdLineArgs>
#include <KIcon>
#include <KStandardDirs>
#include <KUrl>

#include "declarativeviewer.h"
#include "filemodel.h"

// DBUS
#include "applicationadaptor.h"

KLookApp::KLookApp()
    : KUniqueApplication()
    , m_viewer(new DeclarativeViewer())
{
    QString qmlPath = KStandardDirs::locate("appdata", "main.qml");
    if (isLocal()) // this is hack for developers. should replace it with something better I guess
        qmlPath = "../src/qml/main.qml";

    if(!QFile::exists(qmlPath)) {
        qDebug() << "QML file not found";
        exit();
    }

    m_viewer->setSource(QUrl::fromLocalFile(qmlPath));

    qDBusRegisterMetaType<QRect>();

    new ApplicationAdaptor(this);
    QDBusConnection dbus = QDBusConnection::sessionBus();
    dbus.registerObject("/Application", this);
    dbus.registerService("org.kde.klook");
}

KLookApp::~KLookApp()
{
    delete m_viewer->m_fileModel;
}

bool KLookApp::isEmbeddedParam() const
{
    const KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
    return args->isSet("embedded");
}

QRect KLookApp::rectParam() const
{
    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
    QRect rc;
    if (args->isSet("x") && args->isSet("y") &&
            args->isSet("w") && args->isSet("h")) {
        int x, y, w, h;
        x = args->getOption("x").toInt();
        y = args->getOption("y").toInt();
        w = args->getOption("w").toInt();
        h = args->getOption("h").toInt();
        rc = QRect(x, y, w, h);
    }
    return rc;
}

QStringList KLookApp::urlsParam() const
{
    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
    QStringList urls;
    for (int i = 0; i < args->count(); i++) {
        KUrl url(args->arg(i));
        if(url.isRelative()) {
            url.setUrl(args->cwd());
            url.addPath(args->arg(i));
        }
        urls << url.url();
    }
    return urls;
}

int KLookApp::newInstance()
{
    KCmdLineArgs::setCwd(QDir::currentPath().toUtf8());
    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();

    const QStringList urls = urlsParam();
    const QRect rc = rectParam();
    const bool embedded = isEmbeddedParam();

    int index = args->getOption("i").toInt();
    if (index >= urls.count())
        index = 0;

    m_viewer->init(urls, embedded, rc, index);
    args->clear();
    return 0;
}

void KLookApp::hide()
{
    if(m_viewer)
        m_viewer->hide();
}

void KLookApp::showEmbedded(const QRect &rectangle, const QString &url)
{
    m_viewer->init(QStringList() << url, true, rectangle);
}

void KLookApp::showNormal(int x, int y, int width, int height, const QString &url)
{
    m_viewer->init(QStringList() << url, false, QRect(x, y, width, height));
}

bool KLookApp::isLocal() const
{
    const QString appPath = applicationFilePath();
    return !(appPath.startsWith(QLatin1String("/usr/bin")) || appPath.startsWith(QLatin1String("/usr/local/bin")));
}
