/* KLook
 * Copyright (c) 2012 ROSA  <support@rosalab.ru>
 * Authors:
 *  Sergey Borovkov <sergey.borovkov@osinit.ru>
 * License: GPLv3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef LISTITEM_H
#define LISTITEM_H

#include "file.h"

#include <QtCore/QObject>

class ListItemContent;

class ListItem : public QObject
{
    Q_OBJECT
public:
    enum
    {
        FilePathRole = Qt::UserRole + 1,
        // this role will always return url no matter if file is downloaded or not
        UrlRole,
        LoadedRole,
        FileNameRole,
        TypeRole,
        LastModifiedRole,
        ContentSizeRole,
        CountRole,       // object count for folders
        MimeRole,
        LocalFileRole, // is file local??
        DownloadInProgress,
        StatCompleteRole, // for fallback content
        DirSizeCompleteRole, // for dir size job
        ErrorRole
    };

    ListItem(File *file, QObject *parent);

    static QHash<int, QByteArray> roleNames();

    QVariant data(int role) const;

    QString path() const;
    void setPath(QString path)
    {
        m_file->setUrl(QUrl(path));
    }

    QString mime() const
    {
        return m_file->mime();
    }

    int type() const
    {
        return m_file->type();
    }

    void setType(int type)
    {
        m_file->setType(static_cast<File::FileType>(type));
    }

    File *file()
    {
        return m_file;
    }

signals:
    void dataChanged();

private:
    File *m_file;
    mutable ListItemContent *m_content;
};

#endif // LISTITEM_H
