/* KLook
 * Copyright (c) 2012 ROSA  <support@rosalab.ru>
 * Authors:
 *  Sergey Borovkov <sergey.borovkov@osinit.ru>
 * License: GPLv3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#ifndef LISTITEMCONTENT_H
#define LISTITEMCONTENT_H

#include <QtCore/QObject>
#include <QtCore/QDateTime>

class File;
class KJob;

class ListItemContent : public QObject
{
    Q_OBJECT
public:
    explicit ListItemContent(File *file, QObject *parent = 0);
    virtual QVariant data(int role) const = 0;

    File *file() const;

signals:
    void dataChanged();

private:
    File *m_file;
};

class ListItemFallbackContent : public ListItemContent
{
    Q_OBJECT
public:
    explicit ListItemFallbackContent(File *file, QObject *parent = 0);

    virtual QVariant data(int role) const;

private slots:
    void handleStatJob(KJob *job);

private:
    QDateTime m_modificationTime;
    qulonglong m_size;
    bool m_statComplete;
};

class ListItemDirectoryContent : public ListItemFallbackContent
{
    Q_OBJECT
public:
    explicit ListItemDirectoryContent(File *file, QObject *parent = 0);

    virtual QVariant data(int role) const;

private slots:
    void handleDirSizeJob(KJob *job);

private:
    bool m_isScanned;
    qint64 m_totalFiles;
    qint64 m_totalSize;
    QDateTime m_modificationTime;
};

#endif // LISTITEMCONTENT_H
