/* KLook
 * Copyright (c) 2012 ROSA  <support@rosalab.ru>
 * Authors:
 *  Sergey Borovkov <sergey.borovkov@osinit.ru>
 *  Evgeniy Auzhin <evgeniy.augin@osinit.ru>
 * License: GPLv3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef ROTATEDIMAGE_H
#define ROTATEDIMAGE_H

/* Explanation extracted from http://sylvana.net/jpegcrop/exif_orientation.html

   For convenience, here is what the letter F would look like if it were tagged
correctly and displayed by a program that ignores the orientation tag (thus
showing the stored image):

  1        2       3      4         5            6           7          8

888888  888888      88  88      8888888888  88                  88  8888888888
88          88      88  88      88  88      88  88          88  88      88  88
8888      8888    8888  8888    88          8888888888  8888888888          88
88          88      88  88
88          88  888888  888888

*/

#include <QtCore/QSize>
#include <QtCore/QString>
#include <QtGui/QMatrix>

enum Orientation {
    NOT_AVAILABLE = 0,
    NORMAL = 1,
    HFLIP = 2,
    ROT_180 = 3,
    VFLIP = 4,
    TRANSPOSE = 5,
    ROT_90 = 6,
    TRANSVERSE = 7,
    ROT_270 = 8
};

class RotatedImage
{
public:
    RotatedImage(const QString& path);

    QMatrix rotationMatrix() const;

private:
    void rotateImage();
    QMatrix createRotMatrix(int angle);
    QMatrix createScaleMatrix(int dx, int dy);

    QMatrix m_matrix;
    QSize m_size;
    QString m_path;
};

#endif // ROTATEDIMAGE_H
